#Copyright (C) 2008 Codethink Ltd
#Copyright (c) 2012 SUSE LINUX Products GmbH, Nuernberg, Germany.

#This library is free software; you can redistribute it and/or
#modify it under the terms of the GNU Lesser General Public
#License version 2 as published by the Free Software Foundation.

#This program is distributed in the hope that it will be useful,
#but WITHOUT ANY WARRANTY; without even the implied warranty of
#MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#GNU General Public License for more details.
#You should have received a copy of the GNU Lesser General Public License
#along with this program; if not, write to the Free Software
#Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.

from gi.repository import Atspi
from pyatspi.utils import *

__all__ = [
                  "Application",
          ]

#------------------------------------------------------------------------------

class Application:
        """
        An interface identifying an object which is the root of the user
        interface Accessible hierarchy associated with a running application.
        Children of Application are typically, but not exclusively, top-level
        windows.
        """

        def __init__(self, obj):
                self.obj = obj

        def getLocale(self, locale_type):
                """
                Gets the locale in which the application is currently operating.
                For the current message locale, use lctype LOCALE_TYPE_MESSAGES.
                @param : lctype
                The LocaleType for which the locale is queried. 
                @return a string compliant with the POSIX standard for locale
                description.
                """
                print("pyatspi: getLocale unimplemented")
                pass

        def get_id(self):
                return self.obj.get_id()
        _idDoc = \
                """
                The application instance's unique ID as assigned by the registry.
                """
        id = property(fget=get_id, doc=_idDoc)

        def get_toolkitName(self):
                return self.obj.get_toolkit_name()
        _toolkitNameDoc = \
                """
                A string indicating the type of user interface toolkit which
                is used by the application.
                """
        toolkitName = property(fget=get_toolkitName, doc=_toolkitNameDoc)

        def get_version(self):
                return self.obj.get_version()
        _versionDoc = \
                """
                A string indicating the version number of the application's accessibility
                bridge implementation.
                """
        version = property(fget=get_version, doc=_versionDoc)

#END----------------------------------------------------------------------------



#Copyright (c) 2012 SUSE LINUX Products GmbH, Nuernberg, Germany.

#This library is free software; you can redistribute it and/or
#modify it under the terms of the GNU Lesser General Public
#License version 2 as published by the Free Software Foundation.

#This program is distributed in the hope that it will be useful,
#but WITHOUT ANY WARRANTY; without even the implied warranty of
#MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#GNU General Public License for more details.
#You should have received a copy of the GNU Lesser General Public License
#along with this program; if not, write to the Free Software
#Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.

from gi.repository import Atspi
from pyatspi.utils import *
#from pyatspi.component import *

__all__ = [
	   "interface"
]

class interface(object):
	def __init__(self, obj):
		self.obj = obj

	def __eq__(a, b):
		return (a.__class__ == b.__class__ and a.obj == b.obj)



#Copyright (C) 2008 Codethink Ltd
#copyright: Copyright (c) 2005, 2007 IBM Corporation

#This library is free software; you can redistribute it and/or
#modify it under the terms of the GNU Lesser General Public
#License version 2 as published by the Free Software Foundation.

#This program is distributed in the hope that it will be useful,
#but WITHOUT ANY WARRANTY; without even the implied warranty of
#MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#GNU General Public License for more details.
#You should have received a copy of the GNU Lesser General Public License
#along with this program; if not, write to the Free Software
#Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.

#Portions of this code originally licensed and copyright (c) 2005, 2007
#IBM Corporation under the BSD license, available at
#U{http://www.opensource.org/licenses/bsd-license.php}

#authors: Peter Parente, Mark Doffman

from gi.repository import Atspi
from gi.repository import GObject
from pyatspi.atspienum import *

#------------------------------------------------------------------------------

class StateType(AtspiEnum):
        _enum_lookup = {
                0:'STATE_INVALID',
                1:'STATE_ACTIVE',
                2:'STATE_ARMED',
                3:'STATE_BUSY',
                4:'STATE_CHECKED',
                5:'STATE_COLLAPSED',
                6:'STATE_DEFUNCT',
                7:'STATE_EDITABLE',
                8:'STATE_ENABLED',
                9:'STATE_EXPANDABLE',
                10:'STATE_EXPANDED',
                11:'STATE_FOCUSABLE',
                12:'STATE_FOCUSED',
                13:'STATE_HAS_TOOLTIP',
                14:'STATE_HORIZONTAL',
                15:'STATE_ICONIFIED',
                16:'STATE_MODAL',
                17:'STATE_MULTI_LINE',
                18:'STATE_MULTISELECTABLE',
                19:'STATE_OPAQUE',
                20:'STATE_PRESSED',
                21:'STATE_RESIZABLE',
                22:'STATE_SELECTABLE',
                23:'STATE_SELECTED',
                24:'STATE_SENSITIVE',
                25:'STATE_SHOWING',
                26:'STATE_SINGLE_LINE',
                27:'STATE_STALE',
                28:'STATE_TRANSIENT',
                29:'STATE_VERTICAL',
                30:'STATE_VISIBLE',
                31:'STATE_MANAGES_DESCENDANTS',
                32:'STATE_INDETERMINATE',
                33:'STATE_REQUIRED',
                34:'STATE_TRUNCATED',
                35:'STATE_ANIMATED',
                36:'STATE_INVALID_ENTRY',
                37:'STATE_SUPPORTS_AUTOCOMPLETION',
                38:'STATE_SELECTABLE_TEXT',
                39:'STATE_IS_DEFAULT',
                40:'STATE_VISITED',
                41:'STATE_CHECKABLE',
                42:'STATE_HAS_POPUP',
                43:'STATE_READ_ONLY',
                44:'STATE_LAST_DEFINED',
        }

#------------------------------------------------------------------------------

STATE_ACTIVE = StateType(1)
STATE_ANIMATED = StateType(35)
STATE_ARMED = StateType(2)
STATE_BUSY = StateType(3)
STATE_CHECKABLE = StateType(41)
STATE_CHECKED = StateType(4)
STATE_COLLAPSED = StateType(5)
STATE_DEFUNCT = StateType(6)
STATE_EDITABLE = StateType(7)
STATE_ENABLED = StateType(8)
STATE_EXPANDABLE = StateType(9)
STATE_EXPANDED = StateType(10)
STATE_FOCUSABLE = StateType(11)
STATE_FOCUSED = StateType(12)
STATE_HAS_POPUP = StateType(42)
STATE_HAS_TOOLTIP = StateType(13)
STATE_HORIZONTAL = StateType(14)
STATE_ICONIFIED = StateType(15)
STATE_INDETERMINATE = StateType(32)
STATE_INVALID = StateType(0)
STATE_INVALID_ENTRY = StateType(36)
STATE_IS_DEFAULT = StateType(39)
STATE_LAST_DEFINED = StateType(44)
STATE_MANAGES_DESCENDANTS = StateType(31)
STATE_MODAL = StateType(16)
STATE_MULTISELECTABLE = StateType(18)
STATE_MULTI_LINE = StateType(17)
STATE_OPAQUE = StateType(19)
STATE_PRESSED = StateType(20)
STATE_READ_ONLY = StateType(43)
STATE_REQUIRED = StateType(33)
STATE_RESIZABLE = StateType(21)
STATE_SELECTABLE = StateType(22)
STATE_SELECTABLE_TEXT = StateType(38)
STATE_SELECTED = StateType(23)
STATE_SENSITIVE = StateType(24)
STATE_SHOWING = StateType(25)
STATE_SINGLE_LINE = StateType(26)
STATE_STALE = StateType(27)
STATE_SUPPORTS_AUTOCOMPLETION = StateType(37)
STATE_TRANSIENT = StateType(28)
STATE_TRUNCATED = StateType(34)
STATE_VERTICAL = StateType(29)
STATE_VISIBLE = StateType(30)
STATE_VISITED = StateType(40)

#------------------------------------------------------------------------------

# Build a dictionary mapping state values to names based on the prefix of the enum constants.

STATE_VALUE_TO_NAME = dict(((value, name[6:].lower().replace('_', ' '))
                            for name, value
                            in globals().items() 
                            if name.startswith('STATE_')))

#------------------------------------------------------------------------------

def stateset_init(self, *states):
	GObject.GObject.__init__(self)
	list(map(self.add, states))

# TODO: Fix pygobject so that this isn't needed (BGO#646581 may be related)
def StateSet_getStates(self):
        ret = []
        for i in range(0, 64):
                if (self.states & (1 << i)):
                        ret.append(Atspi.StateType(i))
        return ret

StateSet = Atspi.StateSet
StateSet.getStates = StateSet_getStates
StateSet.isEmpty = StateSet.is_empty
StateSet.raw = lambda x: x
StateSet.unref = lambda x: None
StateSet.__init__ = stateset_init



#Copyright (C) 2008 Codethink Ltd

#This library is free software; you can redistribute it and/or
#modify it under the terms of the GNU Lesser General Public
#License version 2 as published by the Free Software Foundation.

#This program is distributed in the hope that it will be useful,
#but WITHOUT ANY WARRANTY; without even the implied warranty of
#MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#GNU General Public License for more details.
#You should have received a copy of the GNU Lesser General Public License
#along with this program; if not, write to the Free Software
#Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.

from gi.repository import Atspi

from pyatspi.registry import *
Registry = Registry()

from pyatspi.application import *
from pyatspi.constants import *
from pyatspi.editabletext import *
from pyatspi.role import *
from pyatspi.state import *
from pyatspi.text import *
from pyatspi.document import *
from pyatspi.utils import *
from pyatspi.action import *
from pyatspi.component import *
from pyatspi.collection import *
from pyatspi.hypertext import *
from pyatspi.image import *
from pyatspi.selection import *
from pyatspi.table import *
from pyatspi.tablecell import *
from pyatspi.value import *
from pyatspi.appevent import *
from pyatspi.interface import *

def Accessible_getitem(self, i):
        len=self.get_child_count()
        if i < 0:
                i = len + i
        if i < 0 or i >= len:
                raise IndexError
        return self.get_child_at_index(i)

def Accessible_str(self):
        '''
        Gets a human readable representation of the accessible.
        
        @return: Role and name information for the accessible
        @rtype: string
        '''
        try:
                return '[%s | %s]' % (self.getRoleName(), self.name)
        except Exception:
                return '[DEAD]'
        
def pointToList(point):
        return (point.x, point.y)

# TODO: Figure out how to override Atspi.Rect constructor and remove this class
def getInterface(func, obj):
        ret = func(obj)
        if ret:
                return ret
        raise NotImplementedError

def getEventType(event):
        try:
                return event.pyType
        except:
                event.pyType = EventType(event.rawType)
                return event.pyType

def DeviceEvent_str(self):
        '''
        Builds a human readable representation of the event.

        @return: Event description
        @rtype: string
        '''
        if self.type == KEY_PRESSED_EVENT:
            kind = 'pressed'
        elif self.type == KEY_RELEASED_EVENT:
            kind = 'released'
        return '''\
%s
\thw_code: %d
\tevent_string: %s
\tmodifiers: %d
\tid: %d
\ttimestamp: %d
\tis_text: %s''' % (kind, self.hw_code, self.event_string, self.modifiers,
                        self.id, self.timestamp, self.is_text)

def Event_str(self):
        '''
        Builds a human readable representation of the event including event type,
        parameters, and source info.

        @return: Event description
        @rtype: string
        '''
        return '%s(%s, %s, %s)\n\tsource: %s\n\thost_application: %s\n\tsender: %s' % \
               (self.type, self.detail1, self.detail2, self.any_data,
                self.source, self.host_application, self.sender)
  
### Accessible ###
Accessible = Atspi.Accessible
Atspi.Accessible.getChildAtIndex = Atspi.Accessible.get_child_at_index
Atspi.Accessible.getAttributes = Atspi.Accessible.get_attributes_as_array
Atspi.Accessible.getApplication = Atspi.Accessible.get_application
Atspi.Accessible.__getitem__ = Accessible_getitem
Atspi.Accessible.__len__ = Atspi.Accessible.get_child_count
Atspi.Accessible.__bool__ = lambda x: True
Atspi.Accessible.__nonzero__ = lambda x: True
Atspi.Accessible.__str__ = Accessible_str
Atspi.Accessible.childCount = property(fget=Atspi.Accessible.get_child_count)
Atspi.Accessible.getChildCount = Atspi.Accessible.get_child_count
Atspi.Accessible.getIndexInParent = Atspi.Accessible.get_index_in_parent
Atspi.Accessible.getLocalizedRoleName = Atspi.Accessible.get_localized_role_name
Atspi.Accessible.getRelationSet = Atspi.Accessible.get_relation_set
Atspi.Accessible.getRole = Atspi.Accessible.get_role
Atspi.Accessible.getRoleName = Atspi.Accessible.get_role_name
Atspi.Accessible.getState = Atspi.Accessible.get_state_set
try:
    del Atspi.Accessible.children
except:
    pass
Atspi.Accessible.description = property(fget=Atspi.Accessible.get_description)
Atspi.Accessible.objectLocale = property(fget=Atspi.Accessible.get_object_locale)
Atspi.Accessible.accessibleId = property(fget=Atspi.Accessible.get_accessible_id)
Atspi.Accessible.name = property(fget=Atspi.Accessible.get_name)
Atspi.Accessible.isEqual = lambda a,b: a == b
Atspi.Accessible.parent = property(fget=Atspi.Accessible.get_parent)
Atspi.Accessible.setCacheMask = Atspi.Accessible.set_cache_mask
Atspi.Accessible.clearCache = Atspi.Accessible.clear_cache

Atspi.Accessible.id = property(fget=Atspi.Accessible.get_id)
Atspi.Accessible.toolkitName = property(fget=Atspi.Accessible.get_toolkit_name)
Atspi.Accessible.toolkitVersion = property(fget=Atspi.Accessible.get_toolkit_version)
Atspi.Accessible.atspiVersion = property(fget=Atspi.Accessible.get_atspi_version)

Atspi.Accessible.queryAction = lambda x: Action(getInterface(Atspi.Accessible.get_action_iface, x))
Atspi.Accessible.queryCollection = lambda x: Collection(getInterface(Atspi.Accessible.get_collection_iface, x))
Atspi.Accessible.queryComponent = lambda x: Component(getInterface(Atspi.Accessible.get_component_iface, x))
Atspi.Accessible.queryDocument = lambda x: Document(getInterface(Atspi.Accessible.get_document_iface, x))
Atspi.Accessible.queryEditableText = lambda x: EditableText(getInterface(Atspi.Accessible.get_editable_text_iface, x))
Atspi.Accessible.queryHyperlink = lambda x: getInterface(Atspi.Accessible.get_hyperlink, x)
Atspi.Accessible.queryHypertext = lambda x: Hypertext(getInterface(Atspi.Accessible.get_hypertext_iface, x))
Atspi.Accessible.queryImage = lambda x: Image(getInterface(Atspi.Accessible.get_image_iface, x))
Atspi.Accessible.querySelection = lambda x: Selection(getInterface(Atspi.Accessible.get_selection_iface, x))
Atspi.Accessible.queryTable = lambda x: Table(getInterface(Atspi.Accessible.get_table_iface, x))
Atspi.Accessible.queryTableCell = lambda x: TableCell(getInterface(Atspi.Accessible.get_table_cell, x))
Atspi.Accessible.queryText = lambda x: Text(getInterface(Atspi.Accessible.get_text_iface, x))
Atspi.Accessible.queryValue = lambda x: Value(getInterface(Atspi.Accessible.get_value_iface, x))

# Doing this here since otherwise we'd have import recursion
interface.queryAction = lambda x: Action(getInterface(Atspi.Accessible.get_action, x.obj))
interface.queryCollection = lambda x: Collection(getInterface(Atspi.Accessible.get_collection, x.obj))
interface.queryComponent = lambda x: Component(getInterface(Atspi.Accessible.get_component, x.obj))
interface.queryDocument = lambda x: Document(getInterface(Atspi.Accessible.get_document, x.obj))
interface.queryEditableText = lambda x: EditableText(getInterface(Atspi.Accessible.get_editable_text, x.obj))
interface.queryHyperlink = lambda x: getInterface(Atspi.Accessible.get_hyperlink, x.obj)
interface.queryHypertext = lambda x: Hypertext(getInterface(Atspi.Accessible.get_hypertext, x.obj))
interface.queryImage = lambda x: Image(getInterface(Atspi.Accessible.get_image, x.obj))
interface.querySelection = lambda x: Selection(getInterface(Atspi.Accessible.get_selection, x.obj))
interface.queryTable = lambda x: Table(getInterface(Atspi.Accessible.get_table, x.obj))
interface.queryTableCell = lambda x: Table(getInterface(Atspi.Accessible.get_table_cell, x.obj))
interface.queryText = lambda x: Text(getInterface(Atspi.Accessible.get_text, x.obj))
interface.queryValue = lambda x: Value(getInterface(Atspi.Accessible.get_value, x.obj))

### hyperlink ###
Hyperlink = Atspi.Hyperlink
Atspi.Hyperlink.getObject = Atspi.Hyperlink.get_object
Atspi.Hyperlink.getURI = Atspi.Hyperlink.get_uri
Atspi.Hyperlink.isValid = Atspi.Hyperlink.is_valid
Atspi.Hyperlink.endIndex = property(fget=Atspi.Hyperlink.get_end_index)
Atspi.Hyperlink.nAnchors = property(fget=Atspi.Hyperlink.get_n_anchors)
Atspi.Hyperlink.startIndex = property(fget=Atspi.Hyperlink.get_start_index)

### DeviceEvent ###
Atspi.DeviceEvent.__str__ = DeviceEvent_str

### event ###
Atspi.Event.host_application = property(fget=lambda x: x.source.get_application())
Atspi.Event.rawType = Atspi.Event.type
Atspi.Event.source_name = property(fget=lambda x: x.source.name)
Atspi.Event.source_role = property(fget=lambda x: x.source.getRole())
Atspi.Event.type = property(fget=getEventType)
Atspi.Event.__str__ = Event_str

### RelationSet ###
Atspi.Relation.getRelationType = Atspi.Relation.get_relation_type
Atspi.Relation.getNTargets = Atspi.Relation.get_n_targets
Atspi.Relation.getTarget = Atspi.Relation.get_target
RELATION_NULL = Atspi.RelationType.NULL
RELATION_LABEL_FOR = Atspi.RelationType.LABEL_FOR
RELATION_LABELLED_BY = Atspi.RelationType.LABELLED_BY
RELATION_CONTROLLER_FOR = Atspi.RelationType.CONTROLLER_FOR
RELATION_CONTROLLED_BY = Atspi.RelationType.CONTROLLED_BY
RELATION_MEMBER_OF = Atspi.RelationType.MEMBER_OF
RELATION_TOOLTIP_FOR = Atspi.RelationType.TOOLTIP_FOR
RELATION_NODE_CHILD_OF = Atspi.RelationType.NODE_CHILD_OF
RELATION_NODE_PARENT_OF = Atspi.RelationType.NODE_PARENT_OF
RELATION_EXTENDED = Atspi.RelationType.EXTENDED
RELATION_FLOWS_TO = Atspi.RelationType.FLOWS_TO
RELATION_FLOWS_FROM = Atspi.RelationType.FLOWS_FROM
RELATION_SUBWINDOW_OF = Atspi.RelationType.SUBWINDOW_OF
RELATION_EMBEDS = Atspi.RelationType.EMBEDS
RELATION_EMBEDDED_BY = Atspi.RelationType.EMBEDDED_BY
RELATION_POPUP_FOR = Atspi.RelationType.POPUP_FOR
RELATION_PARENT_WINDOW_OF = Atspi.RelationType.PARENT_WINDOW_OF
RELATION_DESCRIPTION_FOR = Atspi.RelationType.DESCRIPTION_FOR
RELATION_DESCRIBED_BY = Atspi.RelationType.DESCRIBED_BY
RELATION_DETAILS = Atspi.RelationType.DETAILS
RELATION_DETAILS_FOR = Atspi.RelationType.DETAILS_FOR
RELATION_ERROR_MESSAGE = Atspi.RelationType.ERROR_MESSAGE
RELATION_ERROR_FOR = Atspi.RelationType.ERROR_FOR

# Build a dictionary mapping relation values to names based on the prefix of the enum constants.

RELATION_VALUE_TO_NAME = dict(((value, name[9:].lower().replace('_', ' ')) 
                               for name, value 
                               in globals().items()
                               if name.startswith('RELATION_')))

### ModifierType ###
MODIFIER_SHIFT = Atspi.ModifierType.SHIFT
MODIFIER_SHIFTLOCK = Atspi.ModifierType.SHIFTLOCK
MODIFIER_CONTROL = Atspi.ModifierType.CONTROL
MODIFIER_ALT = Atspi.ModifierType.ALT
MODIFIER_META = Atspi.ModifierType.META
MODIFIER_META2 = Atspi.ModifierType.META2
MODIFIER_META3 = Atspi.ModifierType.META3
MODIFIER_NUMLOCK = Atspi.ModifierType.NUMLOCK

### EventType ###
KEY_PRESSED_EVENT = Atspi.EventType.KEY_PRESSED_EVENT
KEY_RELEASED_EVENT = Atspi.EventType.KEY_RELEASED_EVENT
BUTTON_PRESSED_EVENT = Atspi.EventType.BUTTON_PRESSED_EVENT
BUTTON_RELEASED_EVENT = Atspi.EventType.BUTTON_RELEASED_EVENT

### KeySynthType ###
KEY_PRESS = Atspi.KeySynthType.PRESS
KEY_PRESSRELEASE = Atspi.KeySynthType.PRESSRELEASE
KEY_RELEASE = Atspi.KeySynthType.RELEASE
KEY_STRING = Atspi.KeySynthType.STRING
KEY_SYM = Atspi.KeySynthType.SYM
KEY_LOCKMODIFIERS = Atspi.KeySynthType.LOCKMODIFIERS
KEY_UNLOCKMODIFIERS = Atspi.KeySynthType.UNLOCKMODIFIERS

### cache ###
cache = Atspi.Cache
setTimeout = Atspi.set_timeout



#Copyright (C) 2008 Codethink Ltd
#Copyright (c) 2012 SUSE LINUX Products GmbH, Nuernberg, Germany.

#This library is free software; you can redistribute it and/or
#modify it under the terms of the GNU Lesser General Public
#License version 2 as published by the Free Software Foundation.

#This program is distributed in the hope that it will be useful,
#but WITHOUT ANY WARRANTY; without even the implied warranty of
#MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#GNU General Public License for more details.
#You should have received a copy of the GNU Lesser General Public License
#along with this program; if not, write to the Free Software
#Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.

from gi.repository import Atspi
from pyatspi.utils import *

__all__ = [
           "Image",
          ]

#------------------------------------------------------------------------------

class Image:
        """
        An interface implemented by objects which render image data or
        pictorial information to the screen. When onscreen components
        include graphical information that is not purely intended to
        enhance "3d effect" or visual layout, but which conveys some
        semantic or informational content to the sighted user, they should
        implement Image, and that semantic content should be conveyed
        textually to the extent possible via the image description, as
        well as the Accessible::name and Accessible::description properties.
        """

        def __init__(self, obj):
                self.obj = obj

        def getImageExtents(self, coordType):
                """
                Obtain a bounding box which entirely contains the image contents,
                as displayed on screen. The bounds returned do not account for
                any viewport clipping or the fact that the image may be partially
                or wholly obscured by other onscreen content. 
                The returned values are meaningful only if the Image has
                both STATE_VISIBLE and STATE_SHOWING.
                @param : coordType
                If 0, the returned bounding box position is returned relative
                to the screen; if 1, the bounding box position is returned relative
                to the containing window. 
                @return a BoundingBox enclosing the image's onscreen representation.
                """
                return getBoundingBox(Atspi.Image.get_image_extents(self.obj, coordType))

        def getImagePosition(self, coord_type):
                """
                Get the coordinates of the current image position on screen.
                The returned values are meaningful only if the Image has
                both STATE_VISIBLE and STATE_SHOWING.
                @param : x
                Back-filled with the x coordinate of the onscreen image (i.e.
                the minimum x coordinate) 
                @param : y
                Back-filled with the y coordinate of the onscreen image (i.e.
                the minimum y coordinate) 
                @param : coordType
                If 0, the returned x and y coordinates are returned relative
                to the screen; if 1, they are returned relative to the containing
                window.
                """
                return pointToList(Atspi.Image.get_image_position(self.obj, coord_type))

        def getImageSize(self):
                """
                Obtain the width and height of the current onscreen view of the
                image. The extents returned do not account for any viewport clipping
                or the fact that the image may be partially or wholly obscured
                by other onscreen content. 
                The returned values are meaningful only if the Image has
                both STATE_VISIBLE and STATE_SHOWING.
                @param : width
                Back-filled with the x extents of the onscreen image (i.e. the
                image width in pixels) 
                @param : height
                Back-filled with the y extents of the onscreen image (i.e. the
                image height in pixels)
                """
                return pointToList(Atspi.Image.get_image_size(self.obj))

        def get_imageDescription(self):
                return Atspi.Image.get_image_description(self.obj)
        _imageDescriptionDoc = \
                """
                A UTF-8 string providing a textual description of what is visually
                depicted in the image.
                """
        imageDescription = property(fget=get_imageDescription, doc=_imageDescriptionDoc)

        def get_imageLocale(self):
                return Atspi.Image.get_image_locale(self.obj)
        _imageLocaleDoc = \
                """
                A string corresponding to the POSIX LC_MESSAGES locale used by
                the imageDescription.
                """
        imageLocale = property(fget=get_imageLocale, doc=_imageLocaleDoc)

#END----------------------------------------------------------------------------



#Copyright (C) 2008 Codethink Ltd

#This library is free software; you can redistribute it and/or
#modify it under the terms of the GNU Lesser General Public
#License version 2 as published by the Free Software Foundation.

#This program is distributed in the hope that it will be useful,
#but WITHOUT ANY WARRANTY; without even the implied warranty of
#MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#GNU General Public License for more details.
#You should have received a copy of the GNU Lesser General Public License
#along with this program; if not, write to the Free Software
#Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.

import pyatspi.registry as registry

from pyatspi.atspienum import *

import traceback

#------------------------------------------------------------------------------

class PressedEventType(AtspiEnum):
        _enum_lookup = {
                0:'KEY_PRESSED_EVENT',
                1:'KEY_RELEASED_EVENT',
                2:'BUTTON_PRESSED_EVENT',
                3:'BUTTON_RELEASED_EVENT',
        }

KEY_PRESSED_EVENT = PressedEventType(0)
KEY_RELEASED_EVENT = PressedEventType(1)
BUTTON_PRESSED_EVENT = PressedEventType(2)
BUTTON_RELEASED_EVENT = PressedEventType(3)

#------------------------------------------------------------------------------

class ControllerEventMask(AtspiEnum):
        _enum_lookup = {
                1:'KEY_PRESSED_EVENT_MASK',
                2:'KEY_RELEASED_EVENT_MASK',
                4:'BUTTON_PRESSED_EVENT_MASK',
                8:'BUTTON_RELEASED_EVENT_MASK',
        }

KEY_PRESSED_EVENT_MASK = ControllerEventMask(1)
KEY_RELEASED_EVENT_MASK = ControllerEventMask(2)
BUTTON_PRESSED_EVENT_MASK = ControllerEventMask(4)
BUTTON_RELEASED_EVENT_MASK = ControllerEventMask(8)

#------------------------------------------------------------------------------

class KeyEventType(AtspiEnum):
        _enum_lookup = {
                0:'KEY_PRESSED',
                1:'KEY_RELEASED',
        }
KEY_PRESSED = KeyEventType(0)
KEY_RELEASED = KeyEventType(1)

#------------------------------------------------------------------------------

class KeySynthType(AtspiEnum):
        _enum_lookup = {
                0:'KEY_PRESS',
                1:'KEY_RELEASE',
                2:'KEY_PRESSRELEASE',
                3:'KEY_SYM',
                4:'KEY_STRING',
                5:'KEY_LOCKMODIFIERS',
                6:'KEY_UNLOCKMODIFIERS',
        }

KEY_LOCKMODIFIERS = KeySynthType(5)
KEY_PRESS = KeySynthType(0)
KEY_PRESSRELEASE = KeySynthType(2)
KEY_RELEASE = KeySynthType(1)
KEY_STRING = KeySynthType(4)
KEY_SYM = KeySynthType(3)
KEY_UNLOCKMODIFIERS = KeySynthType(6)

#------------------------------------------------------------------------------

class ModifierType(AtspiEnum):
        _enum_lookup = {
                0:'MODIFIER_SHIFT',
                1:'MODIFIER_SHIFTLOCK',
                2:'MODIFIER_CONTROL',
                3:'MODIFIER_ALT',
                4:'MODIFIER_META',
                5:'MODIFIER_META2',
                6:'MODIFIER_META3',
                7:'MODIFIER_NUMLOCK',
        }

MODIFIER_ALT = ModifierType(3)
MODIFIER_CONTROL = ModifierType(2)
MODIFIER_META = ModifierType(4)
MODIFIER_META2 = ModifierType(5)
MODIFIER_META3 = ModifierType(6)
MODIFIER_NUMLOCK = ModifierType(7)
MODIFIER_SHIFT = ModifierType(0)
MODIFIER_SHIFTLOCK = ModifierType(1)

def allModifiers():
        """
        Generates all possible keyboard modifiers for use with 
        L{registry.Registry.registerKeystrokeListener}.
        """
        mask = 0
        while mask <= (1 << MODIFIER_NUMLOCK):
                yield mask
                mask += 1

#------------------------------------------------------------------------------

class DeviceEvent(list):
        """
        Wraps an AT-SPI device event with a more Pythonic interface. Primarily adds
        a consume attribute which can be used to cease propagation of a device event.

        @ivar consume: Should this event be consumed and not allowed to pass on to
                observers further down the dispatch chain in this process or possibly
                system wide?
        @type consume: boolean
        @ivar type: Kind of event, KEY_PRESSED_EVENT or KEY_RELEASED_EVENT
        @type type: Accessibility.EventType
        @ivar id: Serial identifier for this key event
        @type id: integer
        @ivar hw_code: Hardware scan code for the key
        @type hw_code: integer
        @ivar modifiers: Modifiers held at the time of the key event
        @type modifiers: integer
        @ivar timestamp: Time at which the event occurred relative to some platform
                dependent starting point (e.g. XWindows start time)
        @type timestamp: integer
        @ivar event_string: String describing the key pressed (e.g. keysym)
        @type event_string: string
        @ivar is_text: Is the event representative of text to be inserted (True), or 
                of a control key (False)?
        @type is_text: boolean
        """
        def __new__(cls, type, id, hw_code, modifiers, timestamp, event_string, is_text):
                return list.__new__(cls, (type, id, hw_code, modifiers, timestamp, event_string, is_text))
        def __init__(self, type, id, hw_code, modifiers, timestamp, event_string, is_text):
                list.__init__(self, (type, id, hw_code, modifiers, timestamp, event_string, is_text))
                self.consume = False
        def _get_type(self):
                return self[0]
        def _set_type(self, val):
                self[0] = val
        type = property(fget=_get_type, fset=_set_type)
        def _get_id(self):
                return self[1]
        def _set_id(self, val):
                self[1] = val
        id = property(fget=_get_id, fset=_set_id)
        def _get_hw_code(self):
                return self[2]
        def _set_hw_code(self, val):
                self[2] = val
        hw_code = property(fget=_get_hw_code, fset=_set_hw_code)
        def _get_modifiers(self):
                return self[3]
        def _set_modifiers(self, val):
                self[3] = val
        modifiers = property(fget=_get_modifiers, fset=_set_modifiers)
        def _get_timestamp(self):
                return self[4]
        def _set_timestamp(self, val):
                self[4] = val
        timestamp = property(fget=_get_timestamp, fset=_set_timestamp)
        def _get_event_string(self):
                return self[5]
        def _set_event_string(self, val):
                self[5] = val
        event_string = property(fget=_get_event_string, fset=_set_event_string)
        def _get_is_text(self):
                return self[6]
        def _set_is_text(self, val):
                self[6] = val
        is_text = property(fget=_get_is_text, fset=_set_is_text)

        def __str__(self):
                """
                Builds a human readable representation of the event.

                @return: Event description
                @rtype: string
                """
                import constants
                if self.type == constants.KEY_PRESSED_EVENT:
                        kind = 'pressed'
                elif self.type == constants.KEY_RELEASED_EVENT:
                        kind = 'released'
                return """\
%s
\thw_code: %d
\tevent_string: %s
\tmodifiers: %d
\tid: %d
\ttimestamp: %d
\tis_text: %s""" % (kind, self.hw_code, self.event_string, self.modifiers,
                self.id, self.timestamp, self.is_text)

#------------------------------------------------------------------------------

class EventListenerMode(list):
        def __new__(cls, synchronous, preemptive, global_):
                return list.__new__(cls, (synchronous, preemptive, global_))
        def __init__(self, synchronous, preemptive, global_):
                list.__init__(self, (synchronous, preemptive, global_))
        def _get_synchronous(self):
                return self[0]
        def _set_synchronous(self, val):
                self[0] = val
        synchronous = property(fget=_get_synchronous, fset=_set_synchronous)
        def _get_preemptive(self):
                return self[1]
        def _set_preemptive(self, val):
                self[1] = val
        preemptive = property(fget=_get_preemptive, fset=_set_preemptive)
        def _get_global_(self):
                return self[2]
        def _set_global_(self, val):
                self[2] = val
        global_ = property(fget=_get_global_, fset=_set_global_)

#------------------------------------------------------------------------------

class KeyDefinition(list):
        def __new__(cls, keycode, keysym, keystring, unused):
                return list.__new__(cls, (keycode, keysym, keystring, unused))
        def __init__(self, keycode, keysym, keystring, unused):
                list.__init__(self, (keycode, keysym, keystring, unused))
        def _get_keycode(self):
                return self[0]
        def _set_keycode(self, val):
                self[0] = val
        keycode = property(fget=_get_keycode, fset=_set_keycode)
        def _get_keysym(self):
                return self[1]
        def _set_keysym(self, val):
                self[1] = val
        keysym = property(fget=_get_keysym, fset=_set_keysym)
        def _get_keystring(self):
                return self[2]
        def _set_keystring(self, val):
                self[2] = val
        keystring = property(fget=_get_keystring, fset=_set_keystring)
        def _get_unused(self):
                return self[3]
        def _set_unused(self, val):
                self[3] = val
        unused = property(fget=_get_unused, fset=_set_unused)



#Copyright (C) 210 Novell, Inc.

#This library is free software; you can redistribute it and/or
#modify it under the terms of the GNU Lesser General Public
#License version 2 as published by the Free Software Foundation.

#This program is distributed in the hope that it will be useful,
#but WITHOUT ANY WARRANTY; without even the implied warranty of
#MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#GNU General Public License for more details.
#You should have received a copy of the GNU Lesser General Public License
#along with this program; if not, write to the Free Software
#Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.

__version__ = (1, 9, 0)

import gi
gi.require_version('Atspi', '2.0')
from gi.repository import Atspi

from pyatspi.Accessibility import *

#This is a re-creation of the namespace pollution implemented
#by PyORBit.
import sys
import pyatspi.Accessibility as Accessibility
sys.modules['Accessibility'] = Accessibility



#Copyright (C) 2008 Codethink Ltd
#copyright: Copyright (c) 2005, 2007 IBM Corporation

#This library is free software; you can redistribute it and/or
#modify it under the terms of the GNU Lesser General Public
#License version 2 as published by the Free Software Foundation.

#This program is distributed in the hope that it will be useful,
#but WITHOUT ANY WARRANTY; without even the implied warranty of
#MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#GNU General Public License for more details.
#You should have received a copy of the GNU Lesser General Public License
#along with this program; if not, write to the Free Software
#Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.

#Portions of this code originally licensed and copyright (c) 2005, 2007
#IBM Corporation under the BSD license, available at
#U{http://www.opensource.org/licenses/bsd-license.php}

#authors: Peter Parente, Mark Doffman

# Constants used in the Component interface to get screen coordinates
DESKTOP_COORDS = 0
WINDOW_COORDS = 1

# Constants used to synthesize mouse events
MOUSE_B1P = 'b1p'
MOUSE_B1R = 'b1r'
MOUSE_B1C = 'b1c'
MOUSE_B1D = 'b1d'
MOUSE_B2P = 'b2p'
MOUSE_B2R = 'b2r'
MOUSE_B2C = 'b2c'
MOUSE_B2D = 'b2d'
MOUSE_B3P = 'b3p'
MOUSE_B3R = 'b3r'
MOUSE_B3C = 'b3c'
MOUSE_B3D = 'b3d'
MOUSE_ABS = 'abs'
MOUSE_REL = 'rel'

# events that clear cached properties
CACHE_EVENTS = ['object:property-change:accessible-name',
                'object:property-change:accessible-description',
                'object:property-change:accessible-role',
                'object:property-change:accessible-parent']

CACHE_PROPERTIES = ''

# This was placed into at-spi-corba because it apparently had a bug where
# one could not register for all the subevents of an event given only an
# AT-SPI event class (ie, first part of the event name).  Pyatspi2 does not
# have this issue, but will leave the tree as-is for now, and some programs
# pass it to registerEventListener, so the constant needs to stay.
# Keys are event names having subevents and values are the subevents
# under the key event; handlers *can* be registered for events not in this tree
EVENT_TREE = {
  'terminal':
    ['terminal:line-changed',
     'terminal:columncount-changed',
     'terminal:linecount-changed',
     'terminal:application-changed',
     'terminal:charwidth-changed'
     ],
  'document':
    ['document:load-complete',
     'document:reload',
     'document:load-stopped',
     'document:attributes-changed'
     'document:page-changed'
     ],
  'object': 
    ['object:property-change',
     'object:bounds-changed',
     'object:link-selected',
     'object:state-changed',
     'object:children-changed',
     'object:visible-data-changed',
     'object:selection-changed',
     'object:model-changed',
     'object:active-descendant-changed',
     'object:row-inserted',
     'object:row-reordered',
     'object:row-deleted',
     'object:column-inserted',
     'object:column-reordered',
     'object:column-deleted',
     'object:text-bounds-changed',
     'object:text-selection-changed',
     'object:text-changed',
     'object:text-attributes-changed',
     'object:text-caret-moved',  
     'object:attributes-changed',
     'object:announcement'],
  'object:text-changed' :
    ['object:text-changed:insert',
    'object:text-changed:delete'],
  'object:property-change' :
    ['object:property-change:accessible-parent', 
    'object:property-change:accessible-name',
    'object:property-change:accessible-description',
    'object:property-change:accessible-help-text',
    'object:property-change:accessible-value',
    'object:property-change:accessible-role',
    'object:property-change:accessible-table-caption',
    'object:property-change:accessible-table-column-description',
    'object:property-change:accessible-table-column-header',
    'object:property-change:accessible-table-row-description',
    'object:property-change:accessible-table-row-header',
    'object:property-change:accessible-table-summary'],
  'object:children-changed' :
    ['object:children-changed:add',
    'object:children-changed:remove'],
  'object:state-changed' :
    ['object:state-changed:active',
    'object:state-changed:animated',
    'object:state-changed:armed',
    'object:state-changed:busy',
    'object:state-changed:checkable',
    'object:state-changed:checked',
    'object:state-changed:collapsed',
    'object:state-changed:default',
    'object:state-changed:defunct',
    'object:state-changed:editable',
    'object:state-changed:enabled',
    'object:state-changed:expandable',
    'object:state-changed:expanded',
    'object:state-changed:focusable',
    'object:state-changed:focused',
    'object:state-changed:has-popup',
    'object:state-changed:has-tooltip',
    'object:state-changed:horizontal',
    'object:state-changed:iconified',
    'object:state-changed:indeterminate',
    'object:state-changed:invalid',
    'object:state-changed:invalid-entry',
    'object:state-changed:manages-descendants',
    'object:state-changed:modal',
    'object:state-changed:multi-line',
    'object:state-changed:multiselectable',
    'object:state-changed:opaque',
    'object:state-changed:pressed',
    'object:state-changed:read-only',
    'object:state-changed:resizable',
    'object:state-changed:required',
    'object:state-changed:selectable',
    'object:state-changed:selected',
    'object:state-changed:sensitive',
    'object:state-changed:showing',
    'object:state-changed:selectable-text',
    'object:state-changed:single-line',
    'object:state-changed:stale',
    'object:state-changed:supports-autocompletion',
    'object:state-changed:transient',
    'object:state-changed:truncated',
    'object:state-changed:vertical',
    'object:state-changed:visible',
    'object:state-changed:visited'],
  'mouse' :
    ['mouse:abs',
    'mouse:rel',
    'mouse:button'],
  'mouse:button' :
    ['mouse:button:1p',
    'mouse:button:1r',
    'mouse:button:2p',
    'mouse:button:2r',
    'mouse:button:3p',
    'mouse:button:3r'],
  'screen-reader' :
    ['screen-reader:region-changed',
     'screen-reader:mode-changed'],
  'window' :
    ['window:minimize',
    'window:maximize',
    'window:restore',
    'window:close',
    'window:create',
    'window:reparent',
    'window:desktop-create',
    'window:desktop-destroy',
    'window:activate',
    'window:deactivate',
    'window:raise',
    'window:lower',
    'window:move',
    'window:resize',
    'window:shade',
    'window:unshade',
    'window:restyle'],
  'focus' :
    ['focus:']
}

from pyatspi.state import *



#Copyright (C) 2008 Codethink Ltd
#Copyright (c) 2012 SUSE LINUX Products GmbH, Nuernberg, Germany.

#This library is free software; you can redistribute it and/or
#modify it under the terms of the GNU Lesser General Public
#License version 2 as published by the Free Software Foundation.

#This program is distributed in the hope that it will be useful,
#but WITHOUT ANY WARRANTY; without even the implied warranty of
#MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#GNU General Public License for more details.
#You should have received a copy of the GNU Lesser General Public License
#along with this program; if not, write to the Free Software
#Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.

from gi.repository import Atspi
from pyatspi.atspienum import *
from pyatspi.utils import *
from pyatspi.interface import *

__all__ = [
           "CoordType",
           "XY_SCREEN",
           "XY_WINDOW",
           "XY_PARENT",
           "ComponentLayer",
           "Component",
           "LAYER_BACKGROUND",
           "LAYER_CANVAS",
           "LAYER_INVALID",
           "LAYER_LAST_DEFINED",
           "LAYER_MDI",
           "LAYER_OVERLAY",
           "LAYER_POPUP",
           "LAYER_WIDGET",
           "LAYER_WINDOW",
           "ScrollType",
           "SCROLL_TOP_LEFT",
           "SCROLL_BOTTOM_RIGHT",
           "SCROLL_TOP_EDGE",
           "SCROLL_BOTTOM_EDGE",
           "SCROLL_LEFT_EDGE",
           "SCROLL_RIGHT_EDGE",
           "SCROLL_ANYWHERE",
          ]

#------------------------------------------------------------------------------

class CoordType(AtspiEnum):
        _enum_lookup = {
                0:'XY_SCREEN',
                1:'XY_WINDOW',
                2:'XY_PARENT',
        }

XY_SCREEN = CoordType(0)
XY_WINDOW = CoordType(1)
XY_PARENT = CoordType(2)

#------------------------------------------------------------------------------

class ComponentLayer(AtspiEnum):
        _enum_lookup = {
                0:'LAYER_INVALID',
                1:'LAYER_BACKGROUND',
                2:'LAYER_CANVAS',
                3:'LAYER_WIDGET',
                4:'LAYER_MDI',
                5:'LAYER_POPUP',
                6:'LAYER_OVERLAY',
                7:'LAYER_WINDOW',
                8:'LAYER_LAST_DEFINED',
        }

LAYER_BACKGROUND = ComponentLayer(1)
LAYER_CANVAS = ComponentLayer(2)
LAYER_INVALID = ComponentLayer(0)
LAYER_LAST_DEFINED = ComponentLayer(8)
LAYER_MDI = ComponentLayer(4)
LAYER_OVERLAY = ComponentLayer(6)
LAYER_POPUP = ComponentLayer(5)
LAYER_WIDGET = ComponentLayer(3)
LAYER_WINDOW = ComponentLayer(7)

#------------------------------------------------------------------------------

class ScrollType(AtspiEnum):
        _enum_lookup = {
                0:'SCROLL_TOP_LEFT',
                1:'SCROLL_BOTTOM_RIGHT',
                2:'SCROLL_TOP_EDGE',
                3:'SCROLL_BOTTOM_EDGE',
                4:'SCROLL_LEFT_EDGE',
                5:'SCROLL_RIGHT_EDGE',
                6:'SCROLL_ANYWHERE',
        }

SCROLL_ANYWHERE = ScrollType(6)
SCROLL_BOTTOM_EDGE = ScrollType(3)
SCROLL_BOTTOM_RIGHT = ScrollType(1)
SCROLL_LEFT_EDGE = ScrollType(4)
SCROLL_RIGHT_EDGE = ScrollType(5)
SCROLL_TOP_EDGE = ScrollType(2)
SCROLL_TOP_LEFT = ScrollType(0)

#------------------------------------------------------------------------------

class Component(interface):
        """
        The Component interface is implemented by objects which occupy
        on-screen space, e.g. objects which have onscreen visual representations.
        The methods in Component allow clients to identify where the
        objects lie in the onscreen coordinate system, their relative
        size, stacking order, and position. It also provides a mechanism
        whereby keyboard focus may be transferred to specific user interface
        elements programmatically. This is a 2D API, coordinates of 3D
        objects are projected into the 2-dimensional screen view for
        purposes of this interface.
        """

        def contains(self, x, y, coord_type):
                """
                @return True if the specified point lies within the Component's
                bounding box, False otherwise.
                """
                return Atspi.Component.contains(self.obj, x, y, coord_type)

        def getAccessibleAtPoint(self, x, y, coord_type):
                """
                @return the Accessible child whose bounding box contains the
                specified point.
                """
                return Atspi.Component.get_accessible_at_point(self.obj, x, y, coord_type)

        def getAlpha(self):
                """
                Obtain the alpha value of the component. An alpha value of 1.0
                or greater indicates that the object is fully opaque, and an
                alpha value of 0.0 indicates that the object is fully transparent.
                Negative alpha values have no defined meaning at this time.
                """
                return Atspi.Component.get_alpha(self.obj)

        def getExtents(self, coord_type):
                """
                Obtain the Component's bounding box, in pixels, relative to the
                specified coordinate system. 
                The returned values are meaningful only if the Component has
                both STATE_VISIBLE and STATE_SHOWING.
                @param coord_type
                @return a BoundingBox which entirely contains the object's onscreen
                visual representation.
                """
                return getBoundingBox(Atspi.Component.get_extents(self.obj, coord_type))

        def getLayer(self):
                """
                @return the ComponentLayer in which this object resides.
                """
                return Atspi.Component.get_layer(self.obj)

        def getMDIZOrder(self):
                """
                Obtain the relative stacking order (i.e. 'Z' order) of an object.
                Larger values indicate that an object is on "top" of the stack,
                therefore objects with smaller MDIZOrder may be obscured by objects
                with a larger MDIZOrder, but not vice-versa. 
                @return an integer indicating the object's place in the stacking
                order.
                """
                return Atspi.Component.get_mdi_z_order(self.obj)

        def getPosition(self, coord_type):
                """
                Obtain the position of the current component in the coordinate
                system specified by coord_type. 
                The returned values are meaningful only if the Component has
                both STATE_VISIBLE and STATE_SHOWING.
                @param : coord_type
                @param : x
                an out parameter which will be back-filled with the returned
                x coordinate. 
                @param : y
                an out parameter which will be back-filled with the returned
                y coordinate.
                """
                return pointToList(Atspi.Component.get_position(self.obj, coord_type))

        def getSize(self):
                """
                Obtain the size, in the coordinate system specified by coord_type,
                of the rectangular area which fully contains the object's visual
                representation, without accounting for viewport clipping. 
                The returned values are meaningful only if the Component has
                both STATE_VISIBLE and STATE_SHOWING.
                @param : width
                the object's horizontal extents in the specified coordinate system.
                @param : height
                the object's vertical extents in the specified coordinate system.
                """
                return pointToList(Atspi.Component.get_size(self.obj))

        def grabFocus(self):
                """
                Request that the object obtain keyboard focus.
                @return True if keyboard focus was successfully transferred to
                the Component.
                """
                return Atspi.Component.grab_focus(self.obj)

        def scrollTo(self, scroll_type):
                """
                Makes the object visible on the screen at a given position by
                scrolling all necessary parents.
                @return True if scrolling was successful.
                """
                return Atspi.Component.scroll_to(self.obj, scroll_type)

        def scrollToPoint(self, coord_type, x, y):
                """
                Makes the object visible on the screen at a given position by
                scrolling all necessary parents.
                @return True if scrolling was successful.
                """
                return Atspi.Component.scroll_to_point(self.obj, coord_type, x, y)

#END----------------------------------------------------------------------------



#Copyright (C) 2008 Codethink Ltd

#This library is free software; you can redistribute it and/or
#modify it under the terms of the GNU Lesser General Public
#License version 2 as published by the Free Software Foundation.

#This program is distributed in the hope that it will be useful,
#but WITHOUT ANY WARRANTY; without even the implied warranty of
#MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#GNU General Public License for more details.
#You should have received a copy of the GNU Lesser General Public License
#along with this program; if not, write to the Free Software
#Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.

from pyatspi.atspienum import *

#------------------------------------------------------------------------------

class Role(AtspiEnum):
        _enum_lookup = {
                0:'ROLE_INVALID',
                1:'ROLE_ACCELERATOR_LABEL',
                2:'ROLE_ALERT',
                3:'ROLE_ANIMATION',
                4:'ROLE_ARROW',
                5:'ROLE_CALENDAR',
                6:'ROLE_CANVAS',
                7:'ROLE_CHECK_BOX',
                8:'ROLE_CHECK_MENU_ITEM',
                9:'ROLE_COLOR_CHOOSER',
                10:'ROLE_COLUMN_HEADER',
                11:'ROLE_COMBO_BOX',
                12:'ROLE_DATE_EDITOR',
                13:'ROLE_DESKTOP_ICON',
                14:'ROLE_DESKTOP_FRAME',
                15:'ROLE_DIAL',
                16:'ROLE_DIALOG',
                17:'ROLE_DIRECTORY_PANE',
                18:'ROLE_DRAWING_AREA',
                19:'ROLE_FILE_CHOOSER',
                20:'ROLE_FILLER',
                21:'ROLE_FOCUS_TRAVERSABLE',
                22:'ROLE_FONT_CHOOSER',
                23:'ROLE_FRAME',
                24:'ROLE_GLASS_PANE',
                25:'ROLE_HTML_CONTAINER',
                26:'ROLE_ICON',
                27:'ROLE_IMAGE',
                28:'ROLE_INTERNAL_FRAME',
                29:'ROLE_LABEL',
                30:'ROLE_LAYERED_PANE',
                31:'ROLE_LIST',
                32:'ROLE_LIST_ITEM',
                33:'ROLE_MENU',
                34:'ROLE_MENU_BAR',
                35:'ROLE_MENU_ITEM',
                36:'ROLE_OPTION_PANE',
                37:'ROLE_PAGE_TAB',
                38:'ROLE_PAGE_TAB_LIST',
                39:'ROLE_PANEL',
                40:'ROLE_PASSWORD_TEXT',
                41:'ROLE_POPUP_MENU',
                42:'ROLE_PROGRESS_BAR',
                43:'ROLE_PUSH_BUTTON',
                44:'ROLE_RADIO_BUTTON',
                45:'ROLE_RADIO_MENU_ITEM',
                46:'ROLE_ROOT_PANE',
                47:'ROLE_ROW_HEADER',
                48:'ROLE_SCROLL_BAR',
                49:'ROLE_SCROLL_PANE',
                50:'ROLE_SEPARATOR',
                51:'ROLE_SLIDER',
                52:'ROLE_SPIN_BUTTON',
                53:'ROLE_SPLIT_PANE',
                54:'ROLE_STATUS_BAR',
                55:'ROLE_TABLE',
                56:'ROLE_TABLE_CELL',
                57:'ROLE_TABLE_COLUMN_HEADER',
                58:'ROLE_TABLE_ROW_HEADER',
                59:'ROLE_TEAROFF_MENU_ITEM',
                60:'ROLE_TERMINAL',
                61:'ROLE_TEXT',
                62:'ROLE_TOGGLE_BUTTON',
                63:'ROLE_TOOL_BAR',
                64:'ROLE_TOOL_TIP',
                65:'ROLE_TREE',
                66:'ROLE_TREE_TABLE',
                67:'ROLE_UNKNOWN',
                68:'ROLE_VIEWPORT',
                69:'ROLE_WINDOW',
                70:'ROLE_EXTENDED',
                71:'ROLE_HEADER',
                72:'ROLE_FOOTER',
                73:'ROLE_PARAGRAPH',
                74:'ROLE_RULER',
                75:'ROLE_APPLICATION',
                76:'ROLE_AUTOCOMPLETE',
                77:'ROLE_EDITBAR',
                78:'ROLE_EMBEDDED',
                79:'ROLE_ENTRY',
                80:'ROLE_CHART',
                81:'ROLE_CAPTION',
                82:'ROLE_DOCUMENT_FRAME',
                83:'ROLE_HEADING',
                84:'ROLE_PAGE',
                85:'ROLE_SECTION',
                86:'ROLE_REDUNDANT_OBJECT',
                87:'ROLE_FORM',
                88:'ROLE_LINK',
                89:'ROLE_INPUT_METHOD_WINDOW',
                90:'ROLE_TABLE_ROW',
                91:'ROLE_TREE_ITEM',
                92:'ROLE_DOCUMENT_SPREADSHEET',
                93:'ROLE_DOCUMENT_PRESENTATION',
                94:'ROLE_DOCUMENT_TEXT',
                95:'ROLE_DOCUMENT_WEB',
                96:'ROLE_DOCUMENT_EMAIL',
                97:'ROLE_COMMENT',
                98:'ROLE_LIST_BOX',
                99:'ROLE_GROUPING',
                100:'ROLE_IMAGE_MAP',
                101:'ROLE_NOTIFICATION',
                102:'ROLE_INFO_BAR',
                103:'ROLE_LEVEL_BAR',
                104:'ROLE_TITLE_BAR',
                105:'ROLE_BLOCK_QUOTE',
                106:'ROLE_AUDIO',
                107:'ROLE_VIDEO',
                108:'ROLE_DEFINITION',
                109:'ROLE_ARTICLE',
                110:'ROLE_LANDMARK',
                111:'ROLE_LOG',
                112:'ROLE_MARQUEE',
                113:'ROLE_MATH',
                114:'ROLE_RATING',
                115:'ROLE_TIMER',
                116:'ROLE_STATIC',
                117:'ROLE_MATH_FRACTION',
                118:'ROLE_MATH_ROOT',
                119:'ROLE_SUBSCRIPT',
                120:'ROLE_SUPERSCRIPT',
                121:'ROLE_DESCRIPTION_LIST',
                122:'ROLE_DESCRIPTION_TERM',
                123:'ROLE_DESCRIPTION_VALUE',
                124:'ROLE_FOOTNOTE',
                125:'ROLE_CONTENT_DELETION',
                126:'ROLE_CONTENT_INSERTION',
                127:'ROLE_MARK',
                128:'ROLE_SUGGESTION',
                129:'ROLE_PUSH_BUTTON_MENU',
                129:'ROLE_SWITCH',
                131:'ROLE_LAST_DEFINED',
        }

ROLE_ACCELERATOR_LABEL = Role(1)
ROLE_ALERT = Role(2)
ROLE_ANIMATION = Role(3)
ROLE_APPLICATION = Role(75)
ROLE_ARROW = Role(4)
ROLE_ARTICLE = Role(109)
ROLE_AUDIO = Role(106)
ROLE_AUTOCOMPLETE = Role(76)
ROLE_BLOCK_QUOTE = Role(105)
ROLE_CALENDAR = Role(5)
ROLE_CANVAS = Role(6)
ROLE_CAPTION = Role(81)
ROLE_CHART = Role(80)
ROLE_CHECK_BOX = Role(7)
ROLE_CHECK_MENU_ITEM = Role(8)
ROLE_COLOR_CHOOSER = Role(9)
ROLE_COLUMN_HEADER = Role(10)
ROLE_COMBO_BOX = Role(11)
ROLE_COMMENT = Role(97)
ROLE_CONTENT_DELETION = Role(125)
ROLE_CONTENT_INSERTION = Role(126)
ROLE_DATE_EDITOR = Role(12)
ROLE_DEFINITION = Role(108)
ROLE_DESCRIPTION_LIST = Role(121)
ROLE_DESCRIPTION_TERM = Role(122)
ROLE_DESCRIPTION_VALUE = Role(123)
ROLE_DESKTOP_FRAME = Role(14)
ROLE_DESKTOP_ICON = Role(13)
ROLE_DIAL = Role(15)
ROLE_DIALOG = Role(16)
ROLE_DIRECTORY_PANE = Role(17)
ROLE_DOCUMENT_EMAIL = Role(96)
ROLE_DOCUMENT_FRAME = Role(82)
ROLE_DOCUMENT_PRESENTATION = Role(93)
ROLE_DOCUMENT_SPREADSHEET = Role(92)
ROLE_DOCUMENT_TEXT = Role(94)
ROLE_DOCUMENT_WEB = Role(95)
ROLE_DRAWING_AREA = Role(18)
ROLE_EDITBAR = Role(77)
ROLE_EMBEDDED = Role(78)
ROLE_ENTRY = Role(79)
ROLE_EXTENDED = Role(70)
ROLE_FILE_CHOOSER = Role(19)
ROLE_FILLER = Role(20)
ROLE_FOCUS_TRAVERSABLE = Role(21)
ROLE_FONT_CHOOSER = Role(22)
ROLE_FOOTER = Role(72)
ROLE_FOOTNOTE = Role(124)
ROLE_FORM = Role(87)
ROLE_FRAME = Role(23)
ROLE_GLASS_PANE = Role(24)
ROLE_GROUPING = Role(99)
ROLE_HEADER = Role(71)
ROLE_HEADING = Role(83)
ROLE_HTML_CONTAINER = Role(25)
ROLE_ICON = Role(26)
ROLE_IMAGE = Role(27)
ROLE_IMAGE_MAP = Role(100)
ROLE_INFO_BAR = Role(102)
ROLE_INPUT_METHOD_WINDOW = Role(89)
ROLE_INTERNAL_FRAME = Role(28)
ROLE_INVALID = Role(0)
ROLE_LABEL = Role(29)
ROLE_LANDMARK = Role(110)
ROLE_LAST_DEFINED = Role(131)
ROLE_LAYERED_PANE = Role(30)
ROLE_LEVEL_BAR = Role(103)
ROLE_LINK = Role(88)
ROLE_LIST = Role(31)
ROLE_LIST_BOX = Role(98)
ROLE_LIST_ITEM = Role(32)
ROLE_LOG = Role(111)
ROLE_MARK = Role(127)
ROLE_MARQUEE = Role(112)
ROLE_MATH = Role(113)
ROLE_MATH_FRACTION = Role(117)
ROLE_MATH_ROOT = Role(118)
ROLE_MENU = Role(33)
ROLE_MENU_BAR = Role(34)
ROLE_MENU_ITEM = Role(35)
ROLE_NOTIFICATION = Role(101)
ROLE_OPTION_PANE = Role(36)
ROLE_PAGE = Role(84)
ROLE_PAGE_TAB = Role(37)
ROLE_PAGE_TAB_LIST = Role(38)
ROLE_PANEL = Role(39)
ROLE_PARAGRAPH = Role(73)
ROLE_PASSWORD_TEXT = Role(40)
ROLE_POPUP_MENU = Role(41)
ROLE_PUSH_BUTTON_MENU = Role(129)
ROLE_PROGRESS_BAR = Role(42)
ROLE_PUSH_BUTTON = Role(43)
ROLE_RADIO_BUTTON = Role(44)
ROLE_RADIO_MENU_ITEM = Role(45)
ROLE_RATING = Role(114)
ROLE_REDUNDANT_OBJECT = Role(86)
ROLE_ROOT_PANE = Role(46)
ROLE_ROW_HEADER = Role(47)
ROLE_RULER = Role(74)
ROLE_SCROLL_BAR = Role(48)
ROLE_SCROLL_PANE = Role(49)
ROLE_SECTION = Role(85)
ROLE_SEPARATOR = Role(50)
ROLE_SLIDER = Role(51)
ROLE_SPIN_BUTTON = Role(52)
ROLE_SPLIT_PANE = Role(53)
ROLE_STATIC = Role(116)
ROLE_STATUS_BAR = Role(54)
ROLE_SUBSCRIPT = Role(119)
ROLE_SUGGESTION = Role(128)
ROLE_SUPERSCRIPT = Role(120)
ROLE_SWITCH = Role(130)
ROLE_TABLE = Role(55)
ROLE_TABLE_CELL = Role(56)
ROLE_TABLE_COLUMN_HEADER = Role(57)
ROLE_TABLE_ROW = Role(90)
ROLE_TABLE_ROW_HEADER = Role(58)
ROLE_TEAROFF_MENU_ITEM = Role(59)
ROLE_TERMINAL = Role(60)
ROLE_TEXT = Role(61)
ROLE_TIMER = Role(115)
ROLE_TITLE_BAR = Role(104)
ROLE_TOGGLE_BUTTON = Role(62)
ROLE_TOOL_BAR = Role(63)
ROLE_TOOL_TIP = Role(64)
ROLE_TREE = Role(65)
ROLE_TREE_ITEM = Role(91)
ROLE_TREE_TABLE = Role(66)
ROLE_UNKNOWN = Role(67)
ROLE_VIDEO = Role(107)
ROLE_VIEWPORT = Role(68)
ROLE_WINDOW = Role(69)

ROLE_NAMES = {
        ROLE_INVALID:'invalid',
        ROLE_ACCELERATOR_LABEL:'accelerator label',
        ROLE_ALERT:'alert',
        ROLE_ANIMATION:'animation',
        ROLE_ARROW:'arrow',
        ROLE_CALENDAR:'calendar',
        ROLE_CANVAS:'canvas',
        ROLE_CHECK_BOX:'check box',
        ROLE_CHECK_MENU_ITEM:'check menu item',
        ROLE_COLOR_CHOOSER:'color chooser',
        ROLE_COLUMN_HEADER:'column header',
        ROLE_COMBO_BOX:'combo box',
        ROLE_DATE_EDITOR:'dateeditor',
        ROLE_DESKTOP_ICON:'desktop icon',
        ROLE_DESKTOP_FRAME:'desktop frame',
        ROLE_DIAL:'dial',
        ROLE_DIALOG:'dialog',
        ROLE_DIRECTORY_PANE:'directory pane',
        ROLE_DRAWING_AREA:'drawing area',
        ROLE_FILE_CHOOSER:'file chooser',
        ROLE_FILLER:'filler',
        ROLE_FONT_CHOOSER:'font chooser',
        ROLE_FRAME:'frame',
        ROLE_GLASS_PANE:'glass pane',
        ROLE_HTML_CONTAINER:'html container',
        ROLE_ICON:'icon',
        ROLE_IMAGE:'image',
        ROLE_INTERNAL_FRAME:'internal frame',
        ROLE_LABEL:'label',
        ROLE_LAYERED_PANE:'layered pane',
        ROLE_LIST:'list',
        ROLE_LIST_ITEM:'list item',
        ROLE_MENU:'menu',
        ROLE_MENU_BAR:'menu bar',
        ROLE_MENU_ITEM:'menu item',
        ROLE_OPTION_PANE:'option pane',
        ROLE_PAGE_TAB:'page tab',
        ROLE_PAGE_TAB_LIST:'page tab list',
        ROLE_PANEL:'panel',
        ROLE_PASSWORD_TEXT:'password text',
        ROLE_POPUP_MENU:'popup menu',
        ROLE_PROGRESS_BAR:'progress bar',
        ROLE_PUSH_BUTTON:'push button',
        ROLE_RADIO_BUTTON:'radio button',
        ROLE_RADIO_MENU_ITEM:'radio menu item',
        ROLE_ROOT_PANE:'root pane',
        ROLE_ROW_HEADER:'row header',
        ROLE_SCROLL_BAR:'scroll bar',
        ROLE_SCROLL_PANE:'scroll pane',
        ROLE_SEPARATOR:'separator',
        ROLE_SLIDER:'slider',
        ROLE_SPLIT_PANE:'split pane',
        ROLE_SPIN_BUTTON:'spin button',
        ROLE_STATUS_BAR:'status bar',
        ROLE_TABLE:'table',
        ROLE_TABLE_CELL:'table cell',
        ROLE_TABLE_COLUMN_HEADER:'table column header',
        ROLE_TABLE_ROW_HEADER:'table row header',
        ROLE_TEAROFF_MENU_ITEM:'tear off menu item',
        ROLE_TERMINAL:'terminal',
        ROLE_TEXT:'text',
        ROLE_TOGGLE_BUTTON:'toggle button',
        ROLE_TOOL_BAR:'tool bar',
        ROLE_TOOL_TIP:'tool tip',
        ROLE_TREE:'tree',
        ROLE_TREE_TABLE:'tree table',
        ROLE_UNKNOWN:'unknown',
        ROLE_VIEWPORT:'viewport',
        ROLE_WINDOW:'window',
        ROLE_HEADER:'header',
        ROLE_FOOTER:'footer',
        ROLE_PARAGRAPH:'paragraph',
        ROLE_RULER:'ruler',
        ROLE_APPLICATION:'application',
        ROLE_AUTOCOMPLETE:'autocomplete',
        ROLE_EDITBAR:'edit bar',
        ROLE_EMBEDDED:'embedded component',
        ROLE_ENTRY:'entry',
        ROLE_CHART:'chart',
        ROLE_CAPTION:'caption',
        ROLE_DOCUMENT_FRAME:'document frame',
        ROLE_HEADING:'heading',
        ROLE_PAGE:'page',
        ROLE_SECTION:'section',
        ROLE_REDUNDANT_OBJECT:'redundant object',
        ROLE_FORM:'form',
        ROLE_LINK:'link',
        ROLE_INPUT_METHOD_WINDOW:'input method window',
        ROLE_TABLE_ROW:'table row',
        ROLE_TREE_ITEM:'tree item',
        ROLE_DOCUMENT_SPREADSHEET:'document spreadsheet',
        ROLE_DOCUMENT_PRESENTATION:'document presentation',
        ROLE_DOCUMENT_TEXT:'document text',
        ROLE_DOCUMENT_WEB:'document web',
        ROLE_DOCUMENT_EMAIL:'document email',
        ROLE_COMMENT:'comment',
        ROLE_LIST_BOX:'list box',
        ROLE_GROUPING:'grouping',
        ROLE_IMAGE_MAP:'image map',
        ROLE_NOTIFICATION:'notification',
        ROLE_INFO_BAR:'info bar',
        ROLE_LEVEL_BAR:'level bar',
        ROLE_TITLE_BAR:'title bar',
        ROLE_BLOCK_QUOTE:'block quote',
        ROLE_AUDIO:'audio',
        ROLE_VIDEO:'video',
        ROLE_DEFINITION:'definition',
        ROLE_ARTICLE:'article',
        ROLE_LANDMARK:'landmark',
        ROLE_LOG:'log',
        ROLE_MARQUEE:'marquee',
        ROLE_MATH:'math',
        ROLE_RATING:'rating',
        ROLE_TIMER:'timer',
        ROLE_STATIC:'static',
        ROLE_MATH_FRACTION:'math fraction',
        ROLE_MATH_ROOT: 'math root',
        ROLE_SUBSCRIPT: 'subscript',
        ROLE_SUPERSCRIPT: 'superscript',
        ROLE_CONTENT_DELETION: 'content deletion',
        ROLE_CONTENT_INSERTION: 'content insertion',
        ROLE_MARK: 'mark',
        ROLE_SUGGESTION: 'suggestion',
        ROLE_PUSH_BUTTON_MENU: 'push button menu',
}

#END----------------------------------------------------------------------------
